# Copyright (C) 2008 by Intevation GmbH
#
# Based on code from
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/456195 which
# is Copyright (C) 2006 by Alessandro Budai
#
# This software may be used and distributed according to the terms
# of the Python License 2.3 or newer, see http://www.python.org/license

"""
Variant of httplib.HTTPConnection that connects through a proxy using
the http CONNECT method (that can be used to proxy SSL connections).
"""

import urlparse
import urllib
import httplib
import socket


class HTTPSProxyConnection(httplib.HTTPConnection):

    _ports = {'http' : 80, 'https' : 443}

    debuglevel = 2

    def putrequest(self, method, url, skip_host=0, skip_accept_encoding=0):
        #print "HTTPSProxyConnection.putrequest", method, url
        scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
        host, port = urllib.splitport(netloc)
        if port is None:
            port = self._ports.get(scheme)
            if port is None:
                raise ValueError("Cannot determine port for URL scheme %r"
                                 % scheme)
        self._real_host = host
        self._real_port = port
        httplib.HTTPConnection.putrequest(self, method, url,
                                          skip_host=skip_host,
                                     skip_accept_encoding=skip_accept_encoding)

    def connect(self):
        httplib.HTTPConnection.connect(self)
        #print "HTTPSProxyConnection.connect", self._real_host, self._real_port
        self.send("CONNECT %s:%d HTTP/1.0\r\n\r\n" % (self._real_host,
                                                      self._real_port))
        #expect a HTTP/1.0 200 Connection established
        response = self.response_class(self.sock, strict=self.strict,
                                       method=self._method)
        (version, code, message) = response._read_status()
        #probably here we can handle auth requests...
        if code != 200:
            #proxy returned and error, abort connection, and raise exception
            self.close()
            raise socket.error, ("Proxy connection failed: %d %s"
                                 % (code, message.strip()))
        #eat up header block from proxy....
        while True:
            #should not use directly fp probably
            line = response.fp.readline()
            if line == '\r\n':
                break

        ssl = socket.ssl(self.sock, None, None)
        self.sock = httplib.FakeSocket(self.sock, ssl)
