# Copyright (C) 2007, 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Simple HTTP server with some static data that can be run in a thread"""

import BaseHTTPServer

import inteproxy.httpserver as httpserver


class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_GET(self):
        """Handle a GET request from the server's contents attribute.
        See HTTPServer for more details.
        """
        for path, headers, data in self.server.contents:
            if path == self.path:
                self.send_response(200)
                for header, value in headers:
                    self.send_header(header, value)
                self.send_header("Content-Length", len(data))
                self.end_headers()
                self.wfile.write(data)
                return
        else:
            self.send_error(404)


class HTTPServer(httpserver.HTTPServer):

    """HTTPServer that serves a limited set of predefined contents.

    The contents are described by a list of tuples of the form (PATH,
    HEADERS, DATA), where PATH and DATA are strings describing the path
    on the server and the data found at that path.  HEADERS should be a
    list of (HEADER-KEYWORD, VALUE) pairs describing the HTTP-header
    fields of the resource at PATH.  The Content-Length header field is
    automatically added by the request handler.
    """

    def __init__(self, contents, server_address=("127.0.0.1", 0),
                 RequestHandlerClass=HTTPRequestHandler):
        self.contents = contents
        httpserver.HTTPServer.__init__(self, server_address,
                                       RequestHandlerClass)
